@@ -43,11 +43,13 @@ struct _TaskItemPrivate {
     GTimeVal     urgent_time;
     guint        blink_timer;
     gboolean     mouse_over;
+    gint         monitor;
     WpApplet    *windowPickerApplet;
 };
 
 enum {
     TASK_ITEM_CLOSED_SIGNAL,
+    TASK_ITEM_MONITOR_CHANGED,
     LAST_SIGNAL
 };
 
@@ -78,6 +80,8 @@ static const GtkTargetEntry drag_types[] = {
 
 static const gint n_drag_types = G_N_ELEMENTS(drag_types);
 
+static void task_item_close (TaskItem *item);
+
 static void update_hints (TaskItem *item) {
     GtkWidget *parent, *widget;
     GtkAllocation allocation_parent, allocation_widget;
@@ -255,7 +259,7 @@ static gboolean task_item_draw (
     GdkPixbuf *pbuf;
     area = priv->area;
     pbuf = priv->pixbuf;
-    gint size = MIN (area.height, area.width);
+    gint size = MIN (area.height, area.width) - 8;
     gboolean active = wnck_window_is_active (priv->window);
     /* load the GSettings key for gray icons */
     gboolean icons_greyscale = wp_applet_get_icons_greyscale (priv->windowPickerApplet);
@@ -464,6 +468,64 @@ static void on_window_icon_changed (WnckWindow *window, TaskItem *item) {
     gtk_widget_queue_draw (GTK_WIDGET (item));
 }
 
+static void
+on_window_type_changed (WnckWindow *window,
+                        TaskItem *item)
+{
+    WnckWindowType type;
+
+    if (item->priv->window != window)
+        return;
+
+    type = wnck_window_get_window_type (window);
+
+    if (type == WNCK_WINDOW_DESKTOP ||
+        type == WNCK_WINDOW_DOCK ||
+        type == WNCK_WINDOW_SPLASHSCREEN ||
+        type == WNCK_WINDOW_MENU)
+      {
+          task_item_close (item);
+      }
+}
+
+static gint
+get_window_monitor (WnckWindow *window)
+{
+    gint x;
+    gint y;
+    gint w;
+    gint h;
+    gint window_monitor;
+    GdkScreen *gdk_screen;
+
+    wnck_window_get_geometry (window, &x, &y, &w, &h);
+
+    gdk_screen = gdk_screen_get_default ();
+    window_monitor = gdk_screen_get_monitor_at_point (gdk_screen,
+                                                      x + w / 2,
+                                                      y + h / 2);
+    return window_monitor;
+}
+
+static void
+on_window_geometry_changed (WnckWindow *window,
+                            TaskItem   *item)
+{
+    gint old_monitor;
+    gint window_monitor;
+
+    window_monitor = get_window_monitor (window);
+
+    old_monitor = item->priv->monitor;
+    if (old_monitor != window_monitor)
+      {
+        item->priv->monitor = window_monitor;
+
+        g_signal_emit (item, task_item_signals[TASK_ITEM_MONITOR_CHANGED], 0,
+                       old_monitor);
+      }
+}
+
 static void on_screen_active_window_changed (
     WnckScreen    *screen,
     WnckWindow    *old_window,
@@ -509,23 +571,57 @@ static void on_screen_window_closed (
     TaskItemPrivate *priv = item->priv;
     g_return_if_fail (WNCK_IS_WINDOW (priv->window));
     if (priv->window == window) {
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_window_closed), item);
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_active_window_changed), item);
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_active_workspace_changed), item);
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_window_closed), item);
-        g_signal_handlers_disconnect_by_func (window,
-            G_CALLBACK (on_window_workspace_changed), item);
-        g_signal_handlers_disconnect_by_func (window,
-            G_CALLBACK (on_window_state_changed), item);
-        g_signal_emit (G_OBJECT (item),
-            task_item_signals[TASK_ITEM_CLOSED_SIGNAL], 0);
+        task_item_close (item);
     }
 }
 
+static void
+disconnect_screen (TaskItem *item)
+{
+    WnckScreen *screen;
+
+    screen = item->priv->screen;
+
+    g_signal_handlers_disconnect_by_func (screen, on_screen_window_closed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (screen,
+                                          on_screen_active_window_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (screen,
+                                          on_screen_active_workspace_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (screen,
+                                          on_screen_active_viewport_changed,
+                                          item);
+}
+
+static void
+disconnect_window (TaskItem *item)
+{
+    WnckWindow *window;
+
+    window = item->priv->window;
+
+    g_signal_handlers_disconnect_by_func (window, on_window_workspace_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (window, on_window_state_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (window, on_window_icon_changed, item);
+    g_signal_handlers_disconnect_by_func (window, on_window_type_changed, item);
+    g_signal_handlers_disconnect_by_func (window, on_window_geometry_changed,
+                                          item);
+}
+
+static void
+task_item_close (TaskItem   *item)
+{
+    disconnect_window (item);
+    disconnect_screen (item);
+
+    g_signal_emit (G_OBJECT (item),
+                   task_item_signals[TASK_ITEM_CLOSED_SIGNAL], 0);
+}
+
 static gboolean activate_window (GtkWidget *widget) {
     gint active;
     TaskItemPrivate *priv;
@@ -723,13 +819,8 @@ task_item_dispose (GObject *object)
 {
     TaskItem *task_item = TASK_ITEM (object);
 
-    g_signal_handlers_disconnect_by_func (task_item->priv->screen, on_screen_active_viewport_changed, task_item);
-    g_signal_handlers_disconnect_by_func (task_item->priv->screen, on_screen_active_window_changed, task_item);
-    g_signal_handlers_disconnect_by_func (task_item->priv->screen, on_screen_active_workspace_changed, task_item);
-    g_signal_handlers_disconnect_by_func (task_item->priv->screen, on_screen_window_closed, task_item);
-    g_signal_handlers_disconnect_by_func (task_item->priv->window, on_window_workspace_changed, task_item);
-    g_signal_handlers_disconnect_by_func (task_item->priv->window, on_window_state_changed, task_item);
-    g_signal_handlers_disconnect_by_func (task_item->priv->window, on_window_icon_changed, task_item);
+    disconnect_window (task_item);
+    disconnect_screen (task_item);
 
     G_OBJECT_CLASS (task_item_parent_class)->dispose (object);
 }
@@ -761,6 +852,10 @@ static void task_item_class_init (TaskItemClass *klass) {
         G_STRUCT_OFFSET (TaskItemClass, itemclosed),
         NULL, NULL,
         g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
+    task_item_signals [TASK_ITEM_MONITOR_CHANGED] =
+        g_signal_new ("monitor-changed", TASK_TYPE_ITEM, G_SIGNAL_RUN_LAST,
+                      0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
 static void task_item_init (TaskItem *item) {
@@ -789,6 +884,7 @@ GtkWidget *task_item_new (WpApplet* windowPickerApplet, WnckWindow *window) {
     screen = wnck_window_get_screen (window);
     priv->screen = screen;
     priv->windowPickerApplet = windowPickerApplet;
+    priv->monitor = get_window_monitor (window);
 
     /** Drag and Drop code
      * This item can be both the target and the source of a drag and drop
@@ -848,6 +944,10 @@ GtkWidget *task_item_new (WpApplet* windowPickerApplet, WnckWindow *window) {
         G_CALLBACK (on_window_state_changed), item);
     g_signal_connect (window, "icon-changed",
         G_CALLBACK (on_window_icon_changed), item);
+    g_signal_connect (window, "type-changed",
+        G_CALLBACK (on_window_type_changed), item);
+    g_signal_connect (window, "geometry-changed",
+        G_CALLBACK (on_window_geometry_changed), item);
     g_signal_connect(item, "draw",
         G_CALLBACK(task_item_draw), windowPickerApplet);
     g_signal_connect (item, "button-release-event",
@@ -866,3 +966,8 @@ GtkWidget *task_item_new (WpApplet* windowPickerApplet, WnckWindow *window) {
     task_item_setup_atk (taskItem);
     return item;
 }
+
+gint task_item_get_monitor (TaskItem *item)
+{
+    return item->priv->monitor;
+}
